@@ -66,6 +66,7 @@ impl GLContext {
             inner: Rc::new(InnerGl::load_with(loadfn)),
         };
         gl.bind_vertex_array(&gl.create_vertex_array().unwrap());
+        gl.enable(consts::TEXTURE_CUBE_MAP_SEAMLESS);
         gl
     }
 
@@ -876,6 +877,33 @@ impl GLContext {
         }
     }
 
+    pub fn tex_sub_image_2d_with_u16_data(
+        &self,
+        target: u32,
+        level: u32,
+        x_offset: u32,
+        y_offset: u32,
+        width: u32,
+        height: u32,
+        format: u32,
+        data_type: DataType,
+        pixels: &[u16],
+    ) {
+        unsafe {
+            self.inner.TexSubImage2D(
+                target,
+                level as i32,
+                x_offset as i32,
+                y_offset as i32,
+                width as i32,
+                height as i32,
+                format,
+                data_type.to_const(),
+                pixels.as_ptr() as *const consts::types::GLvoid,
+            );
+        }
+    }
+
     pub fn tex_sub_image_2d_with_u32_data(
         &self,
         target: u32,
@@ -1055,28 +1083,6 @@ impl GLContext {
         }
     }
 
-    pub fn read_pixels(
-        &self,
-        x: u32,
-        y: u32,
-        width: u32,
-        height: u32,
-        format: u32,
-        data_type: DataType,
-    ) {
-        unsafe {
-            self.inner.ReadPixels(
-                x as i32,
-                y as i32,
-                width as i32,
-                height as i32,
-                format,
-                data_type.to_const(),
-                0 as *mut consts::types::GLvoid,
-            );
-        }
-    }
-
     pub fn read_pixels_with_u8_data(
         &self,
         x: u32,
@@ -1100,6 +1106,29 @@ impl GLContext {
         }
     }
 
+    pub fn read_pixels_with_u16_data(
+        &self,
+        x: u32,
+        y: u32,
+        width: u32,
+        height: u32,
+        format: u32,
+        data_type: DataType,
+        dst_data: &mut [u16],
+    ) {
+        unsafe {
+            self.inner.ReadPixels(
+                x as i32,
+                y as i32,
+                width as i32,
+                height as i32,
+                format,
+                data_type.to_const(),
+                dst_data.as_ptr() as *mut consts::types::GLvoid,
+            )
+        }
+    }
+
     pub fn read_pixels_with_f32_data(
         &self,
         x: u32,
@@ -1184,17 +1213,3 @@ impl ShaderType {
         }
     }
 }
-
-impl DataType {
-    fn to_const(&self) -> u32 {
-        match self {
-            DataType::Float => consts::FLOAT,
-            DataType::Byte => consts::BYTE,
-            DataType::UnsignedByte => consts::UNSIGNED_BYTE,
-            DataType::Short => consts::SHORT,
-            DataType::UnsignedShort => consts::UNSIGNED_SHORT,
-            DataType::Int => consts::INT,
-            DataType::UnsignedInt => consts::UNSIGNED_INT,
-        }
-    }
-}
