@@ -44,6 +44,8 @@ pub struct CPUMesh {
     pub indices: Option<Indices>,
     /// The normals of the vertices. Three contiguous floats defines a normal `(x, y, z)`, therefore the length must be divisable by 3.
     pub normals: Option<Vec<f32>>,
+    /// The tangents of the vertices, orthogonal direction to the normal. Three contiguous floats defines a tangent `(x, y, z)`, therefore the length must be divisable by 3.
+    pub tangents: Option<Vec<f32>>,
     /// The uv coordinates of the vertices. Two contiguous floats defines a coordinate `(u, v)`, therefore the length must be divisable by 2.
     pub uvs: Option<Vec<f32>>,
     /// The colors of the vertices. Four contiguous bytes defines a color `(r, g, b, a)`, therefore the length must be divisable by 4.
@@ -52,6 +54,18 @@ pub struct CPUMesh {
 }
 
 impl CPUMesh {
+    ///
+    /// Returns the material for this mesh in the given list of materials. Returns `None` if no suitable material can be found.
+    ///
+    pub fn material<'a>(&self, materials: &'a [CPUMaterial]) -> Option<&'a CPUMaterial> {
+        self.material_name.as_ref().and_then(|name| {
+            materials
+                .iter()
+                .position(|mat| &mat.name == name)
+                .map(|index| &materials[index])
+        })
+    }
+
     ///
     /// Transforms the mesh by the given transformation.
     ///
@@ -73,12 +87,26 @@ impl CPUMesh {
 
         if let Some(ref mut normals) = self.normals {
             for i in 0..normals.len() / 3 {
-                let p = (normal_transform
-                    * vec4(normals[i * 3], normals[i * 3 + 1], normals[i * 3 + 2], 1.0))
-                .truncate();
-                normals[i * 3] = p.x;
-                normals[i * 3 + 1] = p.y;
-                normals[i * 3 + 2] = p.z;
+                let n = normal_transform
+                    * vec4(normals[i * 3], normals[i * 3 + 1], normals[i * 3 + 2], 1.0);
+                normals[i * 3] = n.x;
+                normals[i * 3 + 1] = n.y;
+                normals[i * 3 + 2] = n.z;
+            }
+        }
+
+        if let Some(ref mut tangents) = self.tangents {
+            for i in 0..tangents.len() / 3 {
+                let t = normal_transform
+                    * vec4(
+                        tangents[i * 3],
+                        tangents[i * 3 + 1],
+                        tangents[i * 3 + 2],
+                        1.0,
+                    );
+                tangents[i * 3] = t.x;
+                tangents[i * 3 + 1] = t.y;
+                tangents[i * 3 + 2] = t.z;
             }
         }
     }
@@ -94,12 +122,16 @@ impl CPUMesh {
             -halfsize, halfsize, 0.0,
         ];
         let normals = vec![0.0, 0.0, 1.0, 0.0, 0.0, 1.0, 0.0, 0.0, 1.0, 0.0, 0.0, 1.0];
+        let tangents = vec![
+            1.0, 0.0, 0.0, 1.0, 1.0, 0.0, 0.0, 1.0, 1.0, 0.0, 0.0, 1.0, 1.0, 0.0, 0.0, 1.0,
+        ];
         let uvs = vec![0.0, 0.0, 1.0, 0.0, 1.0, 1.0, 0.0, 1.0];
         CPUMesh {
             name: "square".to_string(),
             indices: Some(Indices::U8(indices)),
             positions,
             normals: Some(normals),
+            tangents: Some(tangents),
             uvs: Some(uvs),
             ..Default::default()
         }
@@ -243,6 +275,7 @@ impl CPUMesh {
             ..Default::default()
         };
         mesh.compute_normals();
+        mesh.compute_tangents().unwrap();
         mesh
     }
 
@@ -359,12 +392,168 @@ impl CPUMesh {
     /// It will override the current normals if they already exist.
     ///
     pub fn compute_normals(&mut self) {
-        self.normals = Some(
-            self.indices
-                .as_ref()
-                .map(|indices| compute_normals_with_indices(&indices.into_u32(), &self.positions))
-                .unwrap_or_else(|| compute_normals(&self.positions)),
-        );
+        let mut normals = vec![0.0f32; self.positions.len()];
+        self.for_each_triangle(|i0, i1, i2| {
+            let p0 = self.position(i0);
+            let p1 = self.position(i1);
+            let p2 = self.position(i2);
+            let normal = (p1 - p0).cross(p2 - p0);
+            normals[i0 * 3] += normal.x;
+            normals[i0 * 3 + 1] += normal.y;
+            normals[i0 * 3 + 2] += normal.z;
+            normals[i1 * 3] += normal.x;
+            normals[i1 * 3 + 1] += normal.y;
+            normals[i1 * 3 + 2] += normal.z;
+            normals[i2 * 3] += normal.x;
+            normals[i2 * 3 + 1] += normal.y;
+            normals[i2 * 3 + 2] += normal.z;
+        });
+
+        self.for_each_vertex(|i| {
+            let normal = vec3(normals[3 * i], normals[3 * i + 1], normals[3 * i + 2]).normalize();
+            normals[3 * i] = normal.x;
+            normals[3 * i + 1] = normal.y;
+            normals[3 * i + 2] = normal.z;
+        });
+        self.normals = Some(normals);
+    }
+
+    ///
+    /// Computes the per vertex tangents and updates the tangents of the mesh.
+    /// It will override the current tangents if they already exist.
+    ///
+    pub fn compute_tangents(&mut self) -> ThreeDResult<()> {
+        if self.normals.is_none() || self.uvs.is_none() {
+            Err(CoreError::FailedComputingTangents)?;
+        }
+        let mut tan1 = vec![vec3(0.0, 0.0, 0.0); self.positions.len() / 3];
+        let mut tan2 = vec![vec3(0.0, 0.0, 0.0); self.positions.len() / 3];
+
+        self.for_each_triangle(|i0, i1, i2| {
+            let a = self.position(i0);
+            let b = self.position(i1);
+            let c = self.position(i2);
+            let uva = self.uv(i0).unwrap();
+            let uvb = self.uv(i1).unwrap();
+            let uvc = self.uv(i2).unwrap();
+
+            let ba = b - a;
+            let ca = c - a;
+
+            let uvba = uvb - uva;
+            let uvca = uvc - uva;
+
+            let d = uvba.x * uvca.y - uvca.x * uvba.y;
+            if d.abs() > 0.00001 {
+                let r = 1.0 / d;
+                let sdir = (ba * uvca.y - ca * uvba.y) * r;
+                let tdir = (ca * uvba.x - ba * uvca.x) * r;
+                tan1[i0] += sdir;
+                tan1[i1] += sdir;
+                tan1[i2] += sdir;
+                tan2[i0] += tdir;
+                tan2[i1] += tdir;
+                tan2[i2] += tdir;
+            }
+        });
+
+        let mut tangents = vec![0.0f32; 4 * self.positions.len() / 3];
+        self.for_each_vertex(|index| {
+            let normal = self.normal(index).unwrap();
+            let t = tan1[index];
+            let tangent = (t - normal * normal.dot(t)).normalize();
+            let handedness = if normal.cross(tangent).dot(tan2[index]) < 0.0 {
+                1.0
+            } else {
+                -1.0
+            };
+            tangents[index * 4] = tangent.x;
+            tangents[index * 4 + 1] = tangent.y;
+            tangents[index * 4 + 2] = tangent.z;
+            tangents[index * 4 + 3] = handedness;
+        });
+
+        self.tangents = Some(tangents);
+        Ok(())
+    }
+
+    ///
+    ///  Iterates over all vertices in this mesh and calls the callback function with the index for each vertex.
+    ///
+    pub fn for_each_vertex(&self, mut callback: impl FnMut(usize)) {
+        for i in 0..self.positions.len() / 3 {
+            callback(i);
+        }
+    }
+
+    ///
+    /// Iterates over all triangles in this mesh and calls the callback function with the three indices, one for each vertex in the triangle.
+    ///
+    pub fn for_each_triangle(&self, mut callback: impl FnMut(usize, usize, usize)) {
+        match self.indices {
+            Some(Indices::U8(ref indices)) => {
+                for face in 0..indices.len() / 3 {
+                    let index0 = indices[face * 3] as usize;
+                    let index1 = indices[face * 3 + 1] as usize;
+                    let index2 = indices[face * 3 + 2] as usize;
+                    callback(index0, index1, index2);
+                }
+            }
+            Some(Indices::U16(ref indices)) => {
+                for face in 0..indices.len() / 3 {
+                    let index0 = indices[face * 3] as usize;
+                    let index1 = indices[face * 3 + 1] as usize;
+                    let index2 = indices[face * 3 + 2] as usize;
+                    callback(index0, index1, index2);
+                }
+            }
+            Some(Indices::U32(ref indices)) => {
+                for face in 0..indices.len() / 3 {
+                    let index0 = indices[face * 3] as usize;
+                    let index1 = indices[face * 3 + 1] as usize;
+                    let index2 = indices[face * 3 + 2] as usize;
+                    callback(index0, index1, index2);
+                }
+            }
+            None => {
+                for face in 0..self.positions.len() / 9 {
+                    callback(face * 3, face * 3 + 1, face * 3 + 2);
+                }
+            }
+        }
+    }
+
+    ///
+    /// Returns the position of the vertex with the given index.
+    ///
+    pub fn position(&self, vertex_index: usize) -> Vec3 {
+        vec3(
+            self.positions[3 * vertex_index],
+            self.positions[3 * vertex_index + 1],
+            self.positions[3 * vertex_index + 2],
+        )
+    }
+
+    ///
+    /// Returns the normal of the vertex with the given index.
+    ///
+    pub fn normal(&self, vertex_index: usize) -> Option<Vec3> {
+        self.normals.as_ref().map(|normals| {
+            vec3(
+                normals[3 * vertex_index],
+                normals[3 * vertex_index + 1],
+                normals[3 * vertex_index + 2],
+            )
+        })
+    }
+
+    ///
+    /// Returns the uv coordinates of the vertex with the given index.
+    ///
+    pub fn uv(&self, vertex_index: usize) -> Option<Vec2> {
+        self.uvs
+            .as_ref()
+            .map(|uvs| vec2(uvs[2 * vertex_index], uvs[2 * vertex_index + 1]))
     }
 
     ///
@@ -447,89 +636,3 @@ impl CPUMesh {
         Ok(())
     }
 }
-
-fn compute_normals_with_indices(indices: &[u32], positions: &[f32]) -> Vec<f32> {
-    let mut normals = vec![0.0f32; positions.len() * 3];
-    for face in 0..indices.len() / 3 {
-        let index0 = indices[face * 3] as usize;
-        let p0 = vec3(
-            positions[index0 * 3],
-            positions[index0 * 3 + 1],
-            positions[index0 * 3 + 2],
-        );
-        let index1 = indices[face * 3 + 1] as usize;
-        let p1 = vec3(
-            positions[index1 * 3],
-            positions[index1 * 3 + 1],
-            positions[index1 * 3 + 2],
-        );
-        let index2 = indices[face * 3 + 2] as usize;
-        let p2 = vec3(
-            positions[index2 * 3],
-            positions[index2 * 3 + 1],
-            positions[index2 * 3 + 2],
-        );
-
-        let normal = (p1 - p0).cross(p2 - p0);
-        normals[index0 * 3] += normal.x;
-        normals[index0 * 3 + 1] += normal.y;
-        normals[index0 * 3 + 2] += normal.z;
-        normals[index1 * 3] += normal.x;
-        normals[index1 * 3 + 1] += normal.y;
-        normals[index1 * 3 + 2] += normal.z;
-        normals[index2 * 3] += normal.x;
-        normals[index2 * 3 + 1] += normal.y;
-        normals[index2 * 3 + 2] += normal.z;
-    }
-
-    for i in 0..normals.len() / 3 {
-        let normal = vec3(normals[3 * i], normals[3 * i + 1], normals[3 * i + 2]).normalize();
-        normals[3 * i] = normal.x;
-        normals[3 * i + 1] = normal.y;
-        normals[3 * i + 2] = normal.z;
-    }
-    normals
-}
-
-fn compute_normals(positions: &[f32]) -> Vec<f32> {
-    let mut normals = vec![0.0f32; positions.len()];
-    for face in 0..positions.len() / 9 {
-        let index0 = face * 3 as usize;
-        let p0 = vec3(
-            positions[index0 * 3],
-            positions[index0 * 3 + 1],
-            positions[index0 * 3 + 2],
-        );
-        let index1 = face * 3 + 1 as usize;
-        let p1 = vec3(
-            positions[index1 * 3],
-            positions[index1 * 3 + 1],
-            positions[index1 * 3 + 2],
-        );
-        let index2 = face * 3 + 2 as usize;
-        let p2 = vec3(
-            positions[index2 * 3],
-            positions[index2 * 3 + 1],
-            positions[index2 * 3 + 2],
-        );
-
-        let normal = (p1 - p0).cross(p2 - p0);
-        normals[index0 * 3] += normal.x;
-        normals[index0 * 3 + 1] += normal.y;
-        normals[index0 * 3 + 2] += normal.z;
-        normals[index1 * 3] += normal.x;
-        normals[index1 * 3 + 1] += normal.y;
-        normals[index1 * 3 + 2] += normal.z;
-        normals[index2 * 3] += normal.x;
-        normals[index2 * 3 + 1] += normal.y;
-        normals[index2 * 3 + 2] += normal.z;
-    }
-
-    for i in 0..normals.len() / 3 {
-        let normal = vec3(normals[3 * i], normals[3 * i + 1], normals[3 * i + 2]).normalize();
-        normals[3 * i] = normal.x;
-        normals[3 * i + 1] = normal.y;
-        normals[3 * i + 2] = normal.z;
-    }
-    normals
-}
