@@ -66,7 +66,7 @@ impl DepthTargetTexture2DArray {
         clear_state: Option<f32>,
         render: F,
     ) -> ThreeDResult<()> {
-        RenderTargetArray::<u8>::new_depth(&self.context, &self)?.write(
+        RenderTargetArray::new_depth(&self.context, &self)?.write(
             &[],
             depth_layer,
             ClearState {
@@ -83,13 +83,15 @@ impl DepthTargetTexture2DArray {
     /// # Errors
     /// Will return an error if the destination is a color texture.
     ///
+    #[deprecated = "Use RenderTarget::copy_from_array or Screen::copy_from_array instead"]
     pub fn copy_to<T: TextureDataType>(
         &self,
         depth_layer: u32,
         destination: CopyDestination<T>,
         viewport: Viewport,
     ) -> ThreeDResult<()> {
-        RenderTargetArray::new_depth(&self.context, &self)?.copy_to(
+        #[allow(deprecated)]
+        RenderTargetArray::new_depth_internal(&self.context, &self)?.copy_to(
             0,
             depth_layer,
             destination,
