@@ -64,33 +64,42 @@ impl DirectionalLight {
         self.light_buffer.update(3, &[0.0]).unwrap();
     }
 
-    pub fn generate_shadow_map<G: Geometry>(
+    pub fn generate_shadow_map(
         &mut self,
-        target: &Vec3,
         frustrum_height: f32,
-        frustrum_depth: f32,
         texture_width: u32,
         texture_height: u32,
-        geometries: &[G],
+        geometries: &[impl Geometry],
     ) -> ThreeDResult<()> {
         let direction = self.direction();
         let up = compute_up_direction(direction);
 
         let viewport = Viewport::new_at_origo(texture_width, texture_height);
+        let mut aabb = AxisAlignedBoundingBox::EMPTY;
+        for geometry in geometries {
+            aabb.expand_with_aabb(&geometry.aabb());
+        }
+        if aabb.is_empty() {
+            return Ok(());
+        }
+        let target = aabb.center();
+        let position = target - direction * aabb.max().distance(aabb.min());
+        let z_far = aabb.distance_max(&position);
+        let z_near = aabb.distance(&position);
         let shadow_camera = Camera::new_orthographic(
             &self.context,
             viewport,
-            target - direction.normalize() * 0.5 * frustrum_depth,
-            *target,
+            position,
+            target,
             up,
             frustrum_height,
-            0.0,
-            frustrum_depth,
+            z_near,
+            z_far,
         )?;
         self.light_buffer
             .update(4, &shadow_matrix(&shadow_camera).to_slice())?;
 
-        let shadow_texture = DepthTargetTexture2D::new(
+        let mut shadow_texture = DepthTargetTexture2D::new(
             &self.context,
             texture_width,
             texture_height,
@@ -110,7 +119,11 @@ impl DirectionalLight {
                 .iter()
                 .filter(|g| shadow_camera.in_frustum(&g.aabb()))
             {
-                geometry.render_forward(&depth_material, &shadow_camera, &Lights::default())?;
+                geometry.render_with_material(
+                    &depth_material,
+                    &shadow_camera,
+                    &Lights::default(),
+                )?;
             }
             Ok(())
         })?;
@@ -140,11 +153,11 @@ impl Light for DirectionalLight {
                 float shadowEnabled{};
                 mat4 shadowMVP{};
             }};
-            vec3 calculate_lighting{}(vec3 surface_color, vec3 position, vec3 normal, float metallic, float roughness, float occlusion)
+            vec3 calculate_lighting{}(vec3 surface_color, vec3 position, vec3 normal, vec3 view_direction, float metallic, float roughness, float occlusion)
             {{
                 if(base{}.intensity > 0.001) {{
                     vec3 light_color = base{}.intensity * base{}.color;
-                    vec3 result = calculate_light(light_color, -direction{}, surface_color, position, normal, metallic, roughness, occlusion);
+                    vec3 result = calculate_light(light_color, -direction{}, surface_color, view_direction, normal, metallic, roughness);
                     if(shadowEnabled{} > 0.5) {{
                         result *= calculate_shadow(shadowMap{}, shadowMVP{}, position);
                     }}
@@ -157,14 +170,13 @@ impl Light for DirectionalLight {
         
         ", i, i, i, i, i, i, i, i, i, i, i, i, i, i)
     }
-    fn use_uniforms(&self, program: &Program, camera: &Camera, i: u32) -> ThreeDResult<()> {
+    fn use_uniforms(&self, program: &Program, i: u32) -> ThreeDResult<()> {
         if let Some(tex) = self.shadow_map() {
             program.use_texture(&format!("shadowMap{}", i), tex)?;
         } else {
             self.context
                 .use_texture_dummy(&program, &format!("shadowMap{}", i))?;
         }
-        program.use_uniform_vec3("eyePosition", camera.position())?;
         program.use_uniform_block(&format!("LightUniform{}", i), self.buffer());
         Ok(())
     }
