@@ -6,7 +6,7 @@ pub struct PositionMaterial {
     pub render_states: RenderStates,
 }
 
-impl ForwardMaterial for PositionMaterial {
+impl Material for PositionMaterial {
     fn fragment_shader_source(&self, _use_vertex_colors: bool, _lights: &Lights) -> String {
         include_str!("shaders/position_material.frag").to_string()
     }
